#!/usr/bin/env python
# -*- coding: utf-8 -*-

import gtk
import re
import pango
import locale
from gettext import gettext as _

markupRe    	= re.compile( '(<u>|</u>|<i>|</i>|<b>|</b>)')
underlineRe 	= re.compile( '(<u>|</u>)')

DAY = _("day")
DAYS = _("days")
HOUR = _("h")
MINUTE = _("m")

black = gtk.gdk.color_parse('#000000')

if(locale.nl_langinfo(locale.D_FMT)[0:2] != '%d'):
	ANOTHER_DAY_FMT = "%b. %d"
else:
	ANOTHER_DAY_FMT = "%d. %b."

PointerCursor = gtk.gdk.Cursor(gtk.gdk.HAND2)

def getDateLong(d):
	return d.strftime( "%%a, %s" % ANOTHER_DAY_FMT)
class EventWidget (gtk.VBox):
	"""
	Borderless window aligning itself to a status Icon
	"""
	row_string_template = '<span %s>%s</span>' #color="%s"
	row_string_template_now = '<b>%s</b>' % row_string_template
	
	def __init__(self, event):
		gtk.VBox.__init__(self)

		name = gtk.Label()
		name.modify_fg(name.state,event.color)
		name.set_alignment(0,0.5)
		name.set_ellipsize(pango.ELLIPSIZE_END)
				
		time_info = gtk.Label()
		time_info.set_alignment(1,0.5)
		
		time_label = gtk.Label()
		time_label.modify_fg(time_label.state,event.color)

		nameEvtBox_inner = gtk.HBox()
		nameEvtBox_inner.set_border_width(2)
		nameEvtBox_inner.set_spacing(4)
		nameEvtBox_inner.pack_start(time_label,False,False)
		nameEvtBox_inner.pack_start(name,True,True)
		nameEvtBox_inner.pack_start(time_info,False,False)

		nameEvtBox = gtk.EventBox()
		nameEvtBox.set_visible_window(False)
		nameEvtBox.add(nameEvtBox_inner)
		
		nameEvtBox.connect('button-press-event',self.on_click_cb)
		nameEvtBox.connect('enter-notify-event',self.on_enter_cb,not event.is_today())
		nameEvtBox.connect('leave-notify-event',self.on_leave_cb,not event.is_today())
		
		hbox1 = gtk.HBox()
		vbox1 = gtk.VBox()
		
		details = gtk.Alignment(.5,.5,1,1)
		details.set_padding(0,0,12,0)
		details.set_data('visibility',False)
				
		#print event
		markup = ""

		time_string = ''
		
		is_all_day = False
		if(event.is_past()):
			markup+= 'strikethrough="true"'
		if(not event.is_confirmed()):
			markup+= 'font-style="italic"'
		if(event.is_all_day()):
			is_all_day = True
			rowString = self.row_string_template % (markup, event.title )
		else:
			
			#█▇▆▅▄▃▂▁■▣▪▬
			time_label.set_markup('<small><b>%s</b></small>'   % (self.getHourMinuteMarkup(event.start_time) ))
			if(event.is_now()):
				rowString = self.row_string_template_now % (markup, event.title )
			else:
				rowString = self.row_string_template % (markup, event.title )
		time_info.set_markup('<small>%s</small>'  % (self.getDurationMarkup(event.duration)))		
		time_info.modify_fg(time_info.state, event.color)

		dtext = []
		dtext.append("%s" % (event.title))

		if(is_all_day):
			if(event.duration.days >= 1):
				dtext.append("<small>%s</small>" % _("<i>until %s (%s)</i>") % (getDateLong(event.end_time),self.getDurationMarkup(event.duration)))
		else:
			dtext.append("<small><i>%s - %s (%s)</i></small>" % (self.getHourMinuteMarkup(event.start_time), self.getHourMinuteMarkup(event.end_time),self.getDurationMarkup(event.duration)))
		
		if(event.place):
			dtext.append("<b>Place:</b> %s" %(event.place))
			pass
		if(event.comment):
			dtext.append("<i>%s</i>" %(event.comment)) 
			pass

		dtext.append(_("in <i><b>%s</b></i>") % (event.cal_name))


		
		l = gtk.Label()
		l.set_alignment(0,0.5)
		l.set_selectable(True)
		l.set_line_wrap_mode(pango.WRAP_WORD_CHAR)
		l.set_line_wrap(True)
		l.modify_fg(details.state,black)

		l.set_markup("\n".join(dtext[1:]))
		details.add(l)
		details.connect("size-allocate",self.details_box_size_allocate)
		
		tooltiptext = "\n  ".join(dtext)
		
		name.set_markup('%s' % rowString)

		
		self.pack_start(nameEvtBox)
		self.pack_start(details)

		self.show_all()

		details.hide()
		if(not event.is_today()):
			time_info.hide()

		try:
			nameEvtBox.set_tooltip_markup(tooltiptext)
		except:
			pass
		
		self.name_label = name
		self.details = details
		self.time_info = time_info
		self.time_label = time_label
		self.nameEvtBox = nameEvtBox
		
		
	def getDurationMarkup(self,td):
		mpart=""
		hpart=""
		dpart=""
		hours,minutes =  map(int,td.__str__()[-7:-3].split(':'))
		if(hours != 0):
			hpart = "%d%s" % (hours,HOUR)
			if(minutes != 0):
				mpart = "%d%s" % (minutes,MINUTE)		
		else:
			if(minutes != 0):
				mpart = "%d%s" % (minutes,MINUTE)
		if(td.days != 0):
			if(td.days == 1):
				dpart = "%d%s " % (td.days,DAY)
			else:
				dpart = "%d%s " % (td.days,DAYS)
		return ("%s%s%s" %(dpart,hpart,mpart)).strip()
		
	def getHourMinuteMarkup(self,dt):
		return dt.strftime(TIME_FMT)
	def getHourMinuteMarkup_ampm(self,dt):
		return dt.strftime(TIME_FMT).replace(':00','').strip()
	def on_enter_cb(self,widget,event,toggle_timeinfo = True):
		
		if(toggle_timeinfo):
			self.time_info.show()
		t = underlineRe.sub('',self.name_label.get_label())
		self.name_label.set_markup("<u>%s</u>" % t)

		self.nameEvtBox.window.set_cursor(PointerCursor)
	
	def on_leave_cb(self,widget,event,toggle_timeinfo = True):
		if(toggle_timeinfo):
			self.time_info.hide()
		t = underlineRe.sub('',self.name_label.get_label())
		self.name_label.set_markup(t)
		self.nameEvtBox.window.set_cursor(None)

		pass
	def on_click_cb(self, widget,event):
		#print "on_click_cb"
		if (event.button == 1):
			if(self.details.get_property('visible')):
				self.details.hide()
			else:
				self.details.show()
		return False

	def details_box_size_allocate(self,widget,rect):
		top,bottom,left,right = widget.get_padding()
		w = rect.width - left - right - 5
		for l in widget.get_children():
			l.set_size_request(w,-1)
		return False
		
	

if(locale.nl_langinfo(locale.T_FMT_AMPM) != ''):
	TIME_FMT = locale.nl_langinfo(locale.T_FMT_AMPM).replace(':%S ','')
	EventWidget.getHourMinuteMarkup = EventWidget.getHourMinuteMarkup_ampm
else:
	TIME_FMT = "%H:%M"
